###################################################
### chunk number 1: 
###################################################
#line 52 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
require("NMOF")


###################################################
### chunk number 2: 
###################################################
#line 63 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
tfRosenbrock


###################################################
### chunk number 3: 
###################################################
#line 67 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
OF <- tfRosenbrock     # see ?testFunctions
size <- 5L             # set dimension
x <- rep.int(1, size)  # the known solution ...
OF(x)                  # ... should give zero


###################################################
### chunk number 4: 
###################################################
#line 75 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
algo <- list(printBar = FALSE,
                   nP = 50L, 
                   nG = 500L,
                    F = 0.6, 
                   CR = 0.9, 
                  min = rep(-100, size), 
                  max = rep( 100, size))


###################################################
### chunk number 5: 
###################################################
#line 85 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
# a vectorised OF: works only with *matrix* x
OF2 <- function(x) {
    n <- dim(x)[1L]
    xi <- x[1L:(n - 1L), ]
    colSums(100 * (x[2L:n, ] - xi * xi)^2 + (1 - xi)^2)
}


###################################################
### chunk number 6: 
###################################################
#line 94 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
x <- matrix(rnorm(size * algo$nP), size, algo$nP)
c(OF(x[ ,1L]), OF(x[ ,2L]), OF(x[ ,3L]))
OF2(x)[1L:3L]  # should give the same result
all.equal(OF2(x)[1L:3L], c(OF(x[ ,1L]), OF(x[ ,2L]), OF(x[ ,3L])))


###################################################
### chunk number 7: 
###################################################
#line 104 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
set.seed(1223445)
(t1 <- system.time(sol <- DEopt(OF = OF, algo = algo)))

algo$loopOF <- FALSE
set.seed(1223445)
(t2 <- system.time(sol2 <- DEopt(OF = OF2, algo = algo)))


###################################################
### chunk number 8: 
###################################################
#line 113 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
sol$OFvalue    # both should be zero (with luck)
sol2$OFvalue    
t1[[3L]]/t2[[3L]]  # speedup


###################################################
### chunk number 9: 
###################################################
#line 140 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
na <- 100L  # number of assets
np <- 100L  # size of population
trials <- seq_len(100L)  # for speed test

# a covariance matrix
Sigma <- array(0.7, dim = c(na, na)); diag(Sigma) <- 1

# set up population
W <- array(runif(na * np), dim = c(na, np))
# budget constraint
scaleFun <- function(x) x/sum(x); W <- apply(W, 2L, scaleFun)


###################################################
### chunk number 10: 
###################################################
#line 154 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
# variant 1
t1 <- system.time({
for (i in trials) {
    res1 <- numeric(np)
    for (j in seq_len(np)) {
        w <- W[ ,j]
        res1[j] <- w %*% Sigma %*% w
    }
}
})

# variant 2
t2 <- system.time({
    for (i in trials) res2 <- diag(t(W) %*% Sigma %*% W)
})

# variant 3
t3 <- system.time({
    for (i in trials) res3 <- colSums(Sigma %*% W * W)
})


###################################################
### chunk number 11: 
###################################################
#line 177 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
all.equal(res1,res2)
all.equal(res2,res3)


###################################################
### chunk number 12: 
###################################################
#line 182 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
## time required
#  ... variant 1
t1
#  ... variant 2
t2
#  ... variant 3
t3


###################################################
### chunk number 13: 
###################################################
#line 206 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
n  <- 100L  # number of observation
p  <- 5L    # number of regressors
np <- 100L  # population size
trials <- seq_len(1000L)  

# random data
X <- array(rnorm(n * p), dim = c(n, p))
y <- rnorm(n)

# random population
Theta <- array(rnorm(p * np), dim = c(p, np))

# empty residuals matrix
R1 <- array(NA, dim = c(n, np))


###################################################
### chunk number 14: 
###################################################
#line 223 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
system.time({
    for (i in trials) 
        for (j in seq_len(np)) 
            R1[ ,j] <- y - X %*% Theta[ ,j]
})
system.time({
    for (i in trials) 
        R2 <- y - X %*% Theta
})


###################################################
### chunk number 15: 
###################################################
#line 237 "d:/Rcompile/CRANpkg/local/2.12/NMOF/inst/doc/vectorise.Rnw"
all.equal(R1, R2)	# ... should be TRUE


